// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tV  : VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tWV : WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;

float4x4 TT ;

//light properties
float3 lDir <string uiname="Light Direction";> = {0, -5, 2};        //light direction in world space
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};
float lPower <String uiname="Power"; float uimin=0.0;> = 25.0;     //shininess of specular highlight

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 tColor <string uiname="Color Transform";>;

struct vs2ps
{
    float4 PosWVP    : POSITION;
    float4 TexCd     : TEXCOORD0;
    float3 LightDirV : TEXCOORD1;
    float2 UV        : TEXCOORD2;
    float3 ViewDirV  : TEXCOORD3;
    float3 Pos       : TEXCOORD4;
};

// --------------------------------------------------------------------------------------------------
// FUNCTIONS:
// --------------------------------------------------------------------------------------------------

#define TWOPI 6.28318531
#define PI    3.14159265

#define gridSpaceX 0.0001
#define gridSpaceY 0.0001

float param1 = 1;
float param2 = 1;
float param3 = 1;
float param4 = 1;

////////////////////////////////////////////////////////////////////////////////
/*Tear Drop/////
x = a(1- cos(u)) sin(u) cos(v)
y = a (1- cos(u)) sin(u) sin(v)
z =  cos(u)
0 <= u <= 2pi, 0 <= v <= pi
*///////////////////////////////////////////////////////////////////////////////

float3 Tear(float u, float v)
{

       float x = param1 * (1- cos(u)) * sin(u) * cos(v);
       float y = param1 * (1- cos(u)) * sin(u) * sin(v);
       float z = cos(u);

      return float3(x, y, z);
}

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS_Tear(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v, pi;

    u = PosO.x  * TWOPI;
    v = PosO.y  * PI;

     Out.UV  = float2(u, v);

    //get position
    PosO.xyz =Tear(u, v);
    PosO.w = 1;

    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(lDir, tV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS_Tear(vs2ps In): COLOR
{
    float u1, v1, u2, v2;
    float3 tang, bitang;
    float3 NormO, NormV;

    //to get neighbour in u direction
    u1 = In.UV.x + gridSpaceX;
    u2 = In.UV.x - gridSpaceX;

    //to get neighbour in v direction
    v1 = In.UV.y + gridSpaceY;
    v2 = In.UV.y - gridSpaceY;

    //get tangents
    tang   = (Tear(u1, In.UV.y) - Tear(u2, In.UV.y));
    bitang = (Tear(In.UV.x, v1) - Tear(In.UV.x, v2));

    //get normal
    NormO = cross(tang, bitang);

    //normal in view space
    NormV = mul(float4(NormO, 0), tWV);
    NormV = normalize(NormV);

    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    lAmb.a = 1;
    //halfvector
    float3 H = normalize(In.ViewDirV + In.LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, In.LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, In.LightDirV) * NormV - In.LightDirV);
    //normalized view direction (view space)
    float3 V = normalize(In.ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= (lAmb + diff) + spec;
    return mul(col, tColor);
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------


technique Tear
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_Tear();
        PixelShader = compile ps_2_a PS_Tear();
    }
}
